<?php

/****************************************************************************
* Project:       PHP Remote Studio Applet
* Invoking a Remote Task in Omnis Studio (Version > 4.1) with a PHP Script
*****************************************************************************
* File:          RSA_App.php
* First created: 2004-xx-xx
* Author:        NN
* Last modified: 2006-06-12
* Modified by:   Michael Hufschmidt
* Copyright (C): Raining Data 2004
*****************************************************************************
* Content / Description / Requirements / Usage:
*
* Definition of the main class "RemoteStudio"
****************************************************************************/

require_once ("rsa_utils.php");
require_once ("RSA_Var.php");

class RSA_App {
  var $mOmnisLibrary;
  var $mOmnisClass;
  var $mOmnisServer;
  var $mServerURL;
  var $mServerScript;
  var $mSock;
  var $mRequestBuffer;
  var $mConnectID;
  var $mError;
  var $mErrorText;
  var $mResult;
  var $mParmCount;
  var $mParms;
  var $mParmChanged;

  function RSA_App() {
    define("DT_NONE",        0);
    define("DT_CHAR",        1);
    define("DT_BINARY",      2);
    define("DT_PICTURE",     3);
    define("DT_BOOLEAN",     4);
    define("DT_REAL",        5);
    define("DT_LIST",        6);
    define("DT_ROW",         7);
    define("DT_INTEGER",     8);
    define("DT_DATETIME",    9);
    define("DT_CHARSIZE",    1); // Unicode
    define("MAXCALL_PARMS", 32);
    $this->mConnectID = 0;
  }

  function loadParams($inifilename) {
    // this method reads the properties of the RemoteStudio class
    // from an ini-file, the filename is provided as a parameter.
    // Example of an  .ini - file:
    /************************************************************************
      ; Ini - File for Remote Studio Applet Demo
      ; Syntax: <Parameter> = <Value>
      ; Parameters are case sensitive
      ; Lines starting with ";" and empty lines are ignored

      WebServerURL = 127.0.0.1
      WebServerScript = /cgi-bin/nph-omniscgi.exe
      ; OmnisServer is optional if on the same PC
      OmnisServer =
      OmnisPort = 5912
      OmnisLibrary = RSA_DEMO
      OmnisClass = rtRemoteStudio
      ************************************************************************/
    if (file_exists($inifilename)) {
      $ini_array = parse_ini_file($inifilename);
      $this->mOmnisLibrary = $ini_array['OmnisLibrary'];
      $this->mOmnisClass = $ini_array['OmnisClass'];
      if (strlen($ini_array['OmnisServer'])) {
        $this->mOmnisServer = $ini_array['OmnisServer'].":".$ini_array['OmnisPort'];
      } else {
        $this->mOmnisServer = $ini_array['OmnisPort'];
      }
      $this->mServerURL = $ini_array['WebServerURL'];
      $this->mServerScript = $ini_array['WebServerScript'];
    }
  }

  function getVersion() {
    return "1.0";
  }

  function open() { // Always good. As I cannot seem to keep the socket alive it needs to
    // open&close on every request
    return true;
  }

  function close() {
    if ($this->mConnectID) { // Close instance on server
      $this->preCall(0);
      $this->addPair("RSrval", "");
      $this->sendReq();
      $this->mConnectID = 0;
      $this->clearError(); // As server probably has nothing to say
    }
    return true;
  }

  function addPair($pPairName, $pPairData) {
    $this->mRequestBuffer = $this->mRequestBuffer.$pPairName.'='.$pPairData.'&';
  }

  function addParm($pRSvar) {
    $this->mParms[$this->mParmCount] = $pRSvar;
    $this->mParmCount++;
    $parmName = "RSparm".strval($this->mParmCount);
    $this->addPair($parmName, $pRSvar->toBase64());
  }

  function setError($id, $str) {
    $this->mError = $id;
    $this->mErrorText = $str;
  }

  function getErrorID() {
    return $this->mError;
  }

  function getErrorText() {
    return $this->mErrorText;
  }

  function clearError() {
    $this->mError = 0;
    $this->mErrorText = "";
  }

  function preCall($pType) {
    for ($n = 0; $n < MAXCALL_PARMS; $n ++) {
      $this->mParms[$n] = 0;
      $this->mParmChanged[$n] = 0;
    }
    $this->mParmCount = 0;
    $this->mRequestBuffer = ""; // Setup request buffer
    $this->addPair("OmnisServer", $this->mOmnisServer);
    $this->addPair("OmnisLibrary", $this->mOmnisLibrary);
    $this->addPair("OmnisClass", $this->mOmnisClass);
    $this->addPair("WebServerUrl", $this->mServerURL);
    $this->addPair("WebServerScript", $this->mServerScript);
    if ($this->mConnectID)
      $this->addPair("ConnectionID", $this->mConnectID);
    $this->addPair("MessageType", strval($pType));
  } // Wrapper function for execute() with no parameters
  function executeNoParams($cmd) {
    $args = NULL;
    return $this->execute($cmd, $args);
  }

  function execute($cmd, & $pArgs) {
    $this->mResult = new RSA_Var;
    $this->clearError();
    $this->preCall(1);
    $this->addPair("RSname", $cmd);
    if (is_null($pArgs))
      $pArgc = 0;
    else
      $pArgc = count($pArgs);
    for ($i = 0; $i < $pArgc; $i ++) {
      $this->addParm($pArgs[$i]);
    }
    $this->addPair("RSrval", "");
    $this->sendReq(); // Copy any changed arguments back into pArgs [would have liked this
    // automatically but taking references fails with the stack vars]
    for ($i = 0; $i < $pArgc; $i ++) {
      if ($this->mParmChanged[$i])
        $pArgs[$i] = $this->mParms[$i];
    }
    $this->mParms = NULL;
    return $this->mResult;
  }

  function sendReq() {
    $this->mSock = fsockopen($this->mServerURL, 80, $this->mError, $this->mErrorText, 30); // Port = 80
    // Timeout = 30 seconds
    if (!$this->mSock)
      return;
    fwrite($this->mSock, "POST $this->mServerScript HTTP/1.0\r\n");
    fwrite($this->mSock, "Connection: Keep-Alive\r\n");
    fwrite($this->mSock, "Content-type: application/x-www-form-urlencoded\r\n");
    fwrite($this->mSock, "Content-length: ".strlen($this->mRequestBuffer)."\r\n");
    fwrite($this->mSock, "Accept: */*\r\n");
    fwrite($this->mSock, "\r\n");
    fwrite($this->mSock, $this->mRequestBuffer);
    $chunks = false;
    while (!feof($this->mSock)) {
      do {
        $str = trim(fgets($this->mSock, 1024));
        if (strpos($str, 'Transfer-Encoding: chunked') === 0) {
          $chunks = true;
        }
      } while (strlen($str) > 0);
      if ($chunks) {
        while (!feof($this->mSock)) {
          $size = hexdec(fgets($this->mSock, 32));
          if ($size > 0) {
            $str = '';
            while ((strlen($str) < $size) && (!feof($this->mSock))) {
              $get = $size -strlen($str);
              $str .= fread($this->mSock, $get);
            }
          }
        }
      } else {
        while (!feof($this->mSock)) {
          $str = fread($this->mSock, 65535);
        }
      }
    } // Decode result string
    $clen = strlen($str);
    if ($clen == 0) {
      $this->setError(1, "No response from server");
    }
    $test = "";
    while ($clen) {
      $nextVar = strpos($str, "@");
      $var = substr($str, 0, $nextVar);
      $str = substr($str, $nextVar +1);
      $clen = $clen - $nextVar -1;
      if (strncasecmp($var, "CONNECTIONID=", 13) == 0) {
        $this->mConnectID = floatval(substr($var, 13));
      } else
        if (strncasecmp($var, "RSRVAL=", 7) == 0) {
          $this->mResult->fromBase64(substr($var, 7));
        } else
          if (strncasecmp($var, "RSPARM", 6) == 0) {
            $eqSign = strpos($var, "=");
            $pnum = substr($var, 6, $eqSign -6);
            $var = substr($var, $eqSign +1);
            $this->mParms[$pnum -1]->fromBase64($var);
            $this->mParmChanged[$pnum -1] = 1;
          }
    }
    fclose($this->mSock);
  }
}
?>






