<?php

/****************************************************************************
* Project:       PHP Remote Studio Applet
* Invoking a Remote Task in Omnis Studio (Version > 4.1) with a PHP Script
*****************************************************************************
* File:          RSA_Var.php
* First created: 2004-xx-xx
* Author:        Raining Data
* Last modified: 2006-06-07
* Modified by:   Michael Hufschmidt
* Copyright (C): Raining Data 2004
*****************************************************************************
* Content / Description / Requirements / Usage:
*
* Definition of the class "RSA_Var" which defines Variable Objects
****************************************************************************/

require_once ("RSA_Datetime.php");
require_once ("RSA_List.php");

class RSA_Var {
  var $mType;
  var $mData;
  var $mLength;

  function RSA_Var() {
    $this->mType = DT_NONE;
  }

  function clear() {
    $this->mData = NULL;
    $this->mType = DT_NONE;
  }

  function setDate($pVal) {
    $this->clear();
    $this->mType = DT_DATETIME;
    $this->mData = $pVal;
  }
  function getDate() {
    if ($this->conv(DT_DATETIME))
      return $this->mData;
    return NULL;
  }

  function setList($pVal, $pIsRow) {
    $this->clear();
    $this->mType = DT_LIST + ($pIsRow ? 1 : 0);
    $this->mData = $pVal;
  }
  function getList() {
    if ($this->mType == DT_LIST || $this->mType == DT_ROW)
      return $this->mData;
    return NULL;
  }

  function setLong($pVal) {
    $this->clear();
    $this->mType = DT_INTEGER;
    $this->mData = intval($pVal);
  }
  function getLong() {
    if ($this->conv(DT_INTEGER))
      return $this->mData;
    return NULL;
  }

  function setReal($pVal) {
    $this->clear();
    $this->mType = DT_REAL;
    $this->mData = floatval($pVal);
  }
  function getReal() {
    if ($this->conv(DT_REAL))
      return $this->mData;
    return NULL;
  }

  function setBool($pVal) {
    $this->clear();
    $this->mType = DT_BOOLEAN;
    $pVal = intval($pVal);
    if ($pVal <> 0)
      $pVal = 1;
    $this->mData = $pVal;
  }
  function getBool() {
    if ($this->conv(DT_CHAR))
      return $this->mData;
    return NULL;
  }

  function getChar() {
    if ($this->conv(DT_CHAR))
      return $this->mData;
    return NULL;
  }

  function getCString() {
    if ($this->conv(DT_CHAR))
      return $this->mData;
    return NULL;
  }

  function getBinary() {
    if ($this->mType == DT_BINARY || $this->mType == DT_PICTURE)
      return $this->mData;
    return NULL;
  }

  function getBinaryLen() {
    if ($this->mType == DT_BINARY || $this->mType == DT_PICTURE)
      return $this->mLength;
    return 0;
  }

  function setBinary($bindata, $len, $type) {
    $this->clear();
    $this->mType = $type;
    $this->mData = $bindata;
    $this->mLength = $len;
  }
  function setChar($bindata, $len) {
    $this->setBinary($bindata, $len * DT_CHARSIZE, DT_CHAR);
  }
  function setCString($bindata) {
    $this->setChar($bindata, strlen($bindata));
  }

  function getFlatLen() {
    $len = 2; // For Type & Spare
    switch ($this->mType) {
      case DT_CHAR :
      case DT_BINARY :
      case DT_PICTURE :
        $len += 4; // For Length;
        $len += $this->mLength;
        break;
      case DT_DATETIME :
        $len += 14; // Extra 2 because there is no short
        break;
      case DT_BOOLEAN :
        $len ++;
        break;
      case DT_REAL :
        $len += 8;
        break;
      case DT_LIST :
      case DT_ROW :
        {
          $len += 8; // Number of rows & columns
          $lst = $this->getList();
          for ($r = 1; $r <= $lst->rowCount(); $r ++) {
            for ($c = 1; $c <= $lst->colCount(); $c ++) {
              $var = $lst->getElement($r, $c);
              $len += $var->getFlatLen();
            }
          }
          break;
        }
      case DT_INTEGER :
        $len += 4;
        break;
    }
    return $len;
  }

  function flatten(& $pData, $offset) {
    $pData[$offset] = $this->mType;
    $offset ++;
    $pData[$offset] = 0;
    $offset ++; // Spare
    switch ($this->mType) {
      case DT_CHAR :
        {
          ctoxlong($pData, $this->mLength, $offset);
          $offset += 4;
          for ($i = 0; $i < $this->mLength; $i ++) {
            $pData[$offset ++] = ord($this->mData[$i]);
          }
          break;
        }
      case DT_BINARY :
      case DT_PICTURE :
        {
          ctoxlong($pData, $this->mLength, $offset);
          $offset += 4;
          for ($i = 0; $i < $this->mLength; $i ++) {
            $pData[$offset ++] = ord($this->mData[$i]);
          }
          $offset += $this->mLength;
          break;
        }
      case DT_DATETIME :
        {
          $dt = $this->getDate();
          ctoxlong($pData, $dt->mYear, $offset);
          $offset += 4;
          $pData[$offset] = $dt->mMonth;
          $offset ++;
          $pData[$offset] = $dt->mDay;
          $offset ++;
          $pData[$offset] = $dt->mHour;
          $offset ++;
          $pData[$offset] = $dt->mMin;
          $offset ++;
          $pData[$offset] = $dt->mSec;
          $offset ++;
          $pData[$offset] = $dt->mHun;
          $offset ++;
          $pData[$offset] = $dt->mDateok;
          $offset ++;
          $pData[$offset] = $dt->mTimeok;
          $offset ++;
          $pData[$offset] = $dt->mSecok;
          $offset ++;
          $pData[$offset] = $dt->mHunok;
          $offset ++;
          break;
        }
      case DT_BOOLEAN :
        if ($this->getBool())
          $pData[$offset] = 1;
        else
          $pData[$offset] = 0;
        $offset ++;
        break;
      case DT_REAL :
        {
          ctoxreal($pData, $this->getReal(), $offset);
          $offset += 8;
          break;
        }
      case DT_LIST :
      case DT_ROW :
        {
          $lst = $this->getList();
          ctoxlong($pData, $lst->rowCount(), $offset);
          $offset += 4;
          ctoxlong($pData, $lst->colCount(), $offset);
          $offset += 4;
          for ($r = 1; $r <= $lst->rowCount(); $r ++) {
            for ($c = 1; $c <= $lst->colCount(); $c ++) {
              $var = $lst->getElement($r, $c);
              $offset = $var->flatten($pData, $offset);
            }
          }
          break;
        }
      case DT_INTEGER :
        {
          ctoxlong($pData, $this->getLong(), $offset);
          $offset += 4;
          break;
        }
    }
    return $offset;
  }

  // Unflatten binary data
  function unflatten(& $bindata, $idx) {
    $type = $bindata[$idx];
    $idx += 2;
    switch ($type) {
      case DT_CHAR :
        $len = cfmxlong(array_slice($bindata, $idx, 4));
        $idx += 4;
        $carr = "";
        for ($n = 0; $n < $len; $n ++) {
          $carr .= chr($bindata[$idx + $n]);
        }
        $this->setChar($carr, $len);
        $idx += $len;
        break;
      case DT_BINARY :
      case DT_PICTURE :
        $len = cfmxlong(array_slice($bindata, $idx, 4));
        $idx += 4;
        $binarydata = "";
        for ($n = 0; $n < $len; $n ++) {
          $binarydata .= pack("C", $bindata[$idx ++]); // Got to be a better way!
        }
        $this->setBinary($binarydata, $len, $type);
        break;
      case DT_DATETIME :
        {
          $dt = new RSA_Datetime;
          $dt->mYear = cfmxlong(array_slice($bindata, $idx, 4));
          $idx += 4;
          $dt->mMonth = $bindata[$idx];
          $idx ++;
          $dt->mDay = $bindata[$idx];
          $idx ++;
          $dt->mHour = $bindata[$idx];
          $idx ++;
          $dt->mMin = $bindata[$idx];
          $idx ++;
          $dt->mSec = $bindata[$idx];
          $idx ++;
          $dt->mHun = $bindata[$idx];
          $idx ++;
          $dt->mDateok = $bindata[$idx];
          $idx ++;
          $dt->mTimeok = $bindata[$idx];
          $idx ++;
          $dt->mSecok = $bindata[$idx];
          $idx ++;
          $dt->Hunok = $bindata[$idx];
          $idx ++;
          $this->setDate($dt);
          break;
        }
      case DT_BOOLEAN :
        $this->setBool($bindata[$idx]);
        $idx ++;
        break;
      case DT_INTEGER :
        $val = cfmxlong(array_slice($bindata, $idx, 4));
        $idx += 4;
        $this->setLong($val);
        break;
      case DT_REAL :
        $val = cfmxreal(array_slice($bindata, $idx, 8));
        $idx += 8;
        $this->setReal($val);
        break;
      case DT_LIST :
      case DT_ROW :
        $row = cfmxlong(array_slice($bindata, $idx, 4));
        $idx += 4;
        $col = cfmxlong(array_slice($bindata, $idx, 4));
        $idx += 4;
        $this->mType = $type;
        $this->mData = new RSA_List($row, $col);
        for ($r = 1; $r <= $row; $r ++) {
          for ($c = 1; $c <= $col; $c ++) {
            $lidx = $this->mData->getElementIdx($r, $c);
            $idx = $this->mData->mVars[$lidx]->unflatten($bindata, $idx);
          }
        }
        break;
      default :
        $this->clear();
        break;
    }
    return $idx;
  }

  function toBase64() {
  // This method sometimes throws in a warning / notice
  // "Undefined offset:  xx in RSA_Var.php on line yyy"
  // therefore ini_set('error_reporting',E_ALL & ~E_NOTICE);
  // has been included in the main program.
    static $to_b64 = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";

    $len = $this->getFlatLen();
    $dataoff = 0;
    $data[0] = 0;
    $this->flatten($data, 0);

    $div = $len / 3;
    $rem = $len % 3;
    $newLen = $div * 4 + ($rem != 0 ? $rem +1 : 0);
    $string = "";
    while ($div > 0) {
      $string .= $to_b64[($data[$dataoff] >> 2) & 0x3f];
      $string .= $to_b64[(($data[$dataoff] << 4) & 0x30) + (($data[$dataoff +1] >> 4) & 0xf)];
      $string .= $to_b64[(($data[$dataoff +1] << 2) & 0x3c) + (($data[$dataoff +2] >> 6) & 0x3)];
      $string .= $to_b64[$data[$dataoff +2] & 0x3f];
      $dataoff += 3;
      $div --;
    }
    switch ($rem) {
      case 2 :
        $string .= $to_b64[($data[$dataoff] >> 2) & 0x3f];
        $string .= $to_b64[(($data[$dataoff] << 4) & 0x30) + (($data[$dataoff +1] >> 4) & 0xf)];
        $string .= $to_b64[($data[$dataoff +1] << 2) & 0x3c];
        $string .= '=';
        break;
      case 1 :
        $string .= $to_b64[($data[$dataoff] >> 2) & 0x3f];
        $string .= $to_b64[($data[$dataoff] << 4) & 0x30];
        $string .= '=';
        $string .= '=';
        break;
    }
    return $string;
  }

  // Converts from Omnisbase64 [not the same as php base64] to an integer array
  function fromBase64(& $string) {
    static $b64 = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
    $this->clear();
    $hexstr = "";
    $count = 0;
    if (strlen($string)) {
      $length = strlen($string);
      $orgLength = $length;
      $tmp = 0;
      while ($length > 0) {
        $skip = strspn(substr($string, $tmp), $b64);
        $count += $skip;
        $length -= $skip;
        $tmp += $skip;
        if ($length > 0) {
          $vrfy = strcspn(substr($string, $tmp), $b64);
          for ($i = 0; $i < $vrfy; $i ++) {
            if ($string[$tmp + $i] == ' ')
              continue;
            if ($string[$tmp + $i] == '=') {
              // we should check if we're close to the end of the string
              $rem = $count % 4;
              // rem must be either 2 or 3, otherwise no '=' should be here
              if ($rem < 2)
                return;
              // end-of-message recognized //
              break;
            } else {
              // Transmission error; RFC tells us to ignore this, but:
              //  - the rest of the message is going to even more corrupt
              //    since we're sliding bits out of place
              // If a message is corrupt, it should be dropped. Period.
              //
              return;
            }
          }
          $length -= $vrfy;
          $tmp += $vrfy;
        }
      }
      if ($count > 0) {
        $qw = 0;
        $tw = 0;
        $tmp = 0;
        $length = $orgLength;
        for ($i = 0; $i < $length; $i ++) {
          $ch = ord($string[$i]);
          if ($ch >= 65 && $ch <= 90)
            $bits = $ch -65;
          else
            if ($ch >= 97 && $ch <= 122)
              $bits = $ch -97 + 26;
            else
              if ($ch >= 48 && $ch <= 57)
                $bits = ($ch -48 + 52);
              else
                if ($ch == 61)
                  break;
                else
                  if ($ch == 32)
                    continue;
                  else
                    $bits = 0;
          switch ($qw ++) {
            case 0 :
              $rc[$tw] = (($bits << 2) & 0xfc);
              break;
            case 1 :
              $rc[$tw] |= (($bits >> 4) & 0x03);
              $rc[$tw +1] = (($bits << 4) & 0xf0);
              break;
            case 2 :
              $rc[$tw +1] |= (($bits >> 2) & 0x0f);
              $rc[$tw +2] = (($bits << 6) & 0xc0);
              break;
            case 3 :
              $rc[$tw +2] |= ($bits & 0x3f);
              break;
          }
          if ($qw == 4) {
            $qw = 0;
            $tw += 3;
          }
        }
        $this->unflatten($rc, 0);
      }
    }
  }

  function conv($pNewType) {
    if ($pNewType == $this->mType)
      return TRUE; // Nothing to do
    switch ($pNewType) {
      case DT_DATETIME :
        return $this->convToDateTime();
      case DT_INTEGER :
        return $this->convToInt();
      case DT_REAL :
        return $this->convToNum();
      case DT_CHAR :
        return $this->convToChar();
      case DT_BOOLEAN :
        {
          if ($this->convToInt()) {
            $this->setBool($this->getLong() != 0);
            return TRUE;
          }
          break;
        }
    }
    return FALSE; // Cannot convert
  }

  function convToInt() {
    switch ($this->mType) {
      case DT_NONE :
        $this->setLong(0);
        return TRUE;
      case DT_CHAR :
        {
          if (is_numeric($this->mData)) {
            $this->setLong(intval($this->mData));
            return TRUE;
          }
          break;
        }
      case DT_BOOLEAN :
        $this->setLong(($this->getBool() ? 1 : 0));
        return TRUE;
      case DT_REAL :
        $this->setLong(intval($this->getReal()));
        return TRUE;
      case DT_DATETIME :
        {
          $dt = $this->getDate();
          $val = mktime($dt->mHour, $dt->mMin, $dt->mSec, $dt->mMonth, $dt->mDay, $dt->mYear);
          $this->setLong($val);
          return TRUE;
        }
    }
    return FALSE;
  }

  function convToNum() {
    switch ($this->mType) {
      case DT_NONE :
        $this->setReal(0.0);
        return TRUE;
      case DT_CHAR :
        {
          if (is_float($this->mData)) {
            $this->setReal(floatval($this->mData));
            return TRUE;
          }
          break;
        }
      case DT_BOOLEAN :
        $this->setReal(($this->getBool() ? 1.0 : 0.0));
        return TRUE;
      case DT_INTEGER :
        $this->setReal(floatval($this->getLong()));
        return TRUE;
      case DT_DATETIME :
        {
          $dt = $this->getDate();
          $val = mktime($dt->mHour, $dt->mMin, $dt->mSec, $dt->mMonth, $dt->mDay, $dt->mYear);
          $this->setReal($val);
          return TRUE;
        }
    }
    return FALSE;
  }

  function convToChar() {
    switch ($this->mType) {
      case DT_NONE :
        $this->setCString("");
        return TRUE;
      case DT_CHAR :
        {
          if (is_float($this->mData)) {
            $this->setCString(floatval($this->mData));
            return TRUE;
          }
          break;
        }
      case DT_BOOLEAN :
        $this->setCString(strval($this->getBool()));
        return TRUE;
      case DT_INTEGER :
        $this->setCString(strval($this->getLong()));
        return TRUE;
      case DT_REAL :
        $this->setCString(strval($this->getReal()));
        return TRUE;
      case DT_DATETIME :
        {
          $str = "";
          $dt = $this->getDate();
          $val = mktime($dt->mHour, $dt->mMin, $dt->mSec, $dt->mMonth, $dt->mDay, $dt->mYear);
          if ($dt->mDateok) {
            $str = date("d M Y", $val); // Format is 01 JAN 2003
          }
          if ($dt->mTimeok) {
            if ($str <> "")
              $str .= "  ";
            $str .= date("H:i:s", $val); // Format is 03:01:03
          }
          $this->setCString($str);
          return TRUE;
        }
    }
    return FALSE;
  }

  function convToDateTime() {
    $ok = FALSE;
    $val = 0;
    switch ($this->mType) {
      case DT_NONE :
        $ok = TRUE;
        $val = time();
        break;
      case DT_CHAR :
        {
          $val = strtotime($this->getCString());
          if (-1 != $val)
            $ok = TRUE;
          break;
        }
      case DT_BOOLEAN :
        $ok = TRUE;
        $val = $this->getBool();
        break;
      case DT_INTEGER :
        $ok = TRUE;
        $val = $this->getLong();
        break;
      case DT_REAL :
        $ok = TRUE;
        $val = $this->getReal();
        break;
    }
    if ($ok) {
      $datearray = getdate($val);
      $dt = new RSdatetime;
      $dt->mMonth = $datearray[mon];
      $dt->mDay = $datearray[mday];
      $dt->mHour = $datearray[hours];
      $dt->mMin = $datearray[minutes];
      $dt->mSec = $datearray[seconds];
      $dt->mHun = 0;
      $dt->mDateok = TRUE;
      $dt->mTimeok = TRUE;
      $dt->mSecok = TRUE;
      $dt->Hunok = FALSE;
      $this->setDate($dt);
    }
    return $ok;
  }
}
?>

